@@ -28,7 +28,7 @@
 #include "htmlfontmanager.h"
 
 #define DEFAULT_FONT_SIZE   12
-#define DEFAULT_FONT_SIZE_S "12"
+#define DEFAULT_FONT_SIZE_PRINT   10
 
 #define STRINGIZE(x) #x
 
@@ -51,10 +51,10 @@ gtk_html_class_properties_new (void)
 	p->font_fix_size           = DEFAULT_FONT_SIZE;
 	p->font_var_points         = FALSE;
 	p->font_fix_points         = FALSE;
-	p->font_var_print          = g_strdup ("-*-helvetica-*-*-*-*-12-*-*-*-*-*-*-*");
-	p->font_fix_print          = g_strdup ("-*-courier-*-*-*-*-12-*-*-*-*-*-*-*");
-	p->font_var_size_print     = DEFAULT_FONT_SIZE;
-	p->font_fix_size_print     = DEFAULT_FONT_SIZE;
+	p->font_var_print          = g_strdup ("-*-helvetica-*-*-*-*-10-*-*-*-*-*-*-*");
+	p->font_fix_print          = g_strdup ("-*-courier-*-*-*-*-10-*-*-*-*-*-*-*");
+	p->font_var_size_print     = DEFAULT_FONT_SIZE_PRINT;
+	p->font_fix_size_print     = DEFAULT_FONT_SIZE_PRINT;
 	p->font_var_print_points   = FALSE;
 	p->font_fix_print_points   = FALSE;
 	p->animations              = TRUE;
@@ -181,6 +181,92 @@ gtk_html_class_properties_update (GtkHTMLClassProperties *p, GConfClient *client
         g_free (p->v); \
         GET(string,v,s)
 
+void
+gtk_html_class_properties_load (GtkHTMLClassProperties *p)
+{
+	gchar *s, *var_name, *fix_name, *var_default, *fix_default;
+
+	get_default_fonts (&var_name, &fix_name);
+	var_default = g_strdup_printf ("font_variable=%s", var_name);
+	fix_default = g_strdup_printf ("font_fixed=%s", fix_name);
+	g_free (var_name);
+	g_free (fix_name);
+
+	gnome_config_push_prefix (GTK_HTML_GNOME_CONFIG_PREFIX);
+	GET  (bool, magic_links, "magic_links=true");
+	GET  (bool, animations, "animations=true");
+	GETS (keybindings_theme, "keybindings_theme=ms");
+	GETS (font_var, var_default);
+	GETS (font_fix, fix_default);
+	GETS (font_var_print, "font_variable_print=-*-helvetica-*-*-*-*-10-*-*-*-*-*-*-*");
+	GETS (font_fix_print, "font_fixed_print=-*-courier-*-*-*-*-10-*-*-*-*-*-*-*");
+
+	g_free (var_default);
+	g_free (fix_default);
+
+	s = g_strdup_printf ("font_variable_size=%d", DEFAULT_FONT_SIZE);
+	GET  (int, font_var_size, s);
+	g_free (s);
+
+	s = g_strdup_printf ("font_fixed_size=%d", DEFAULT_FONT_SIZE);
+	GET  (int, font_fix_size, s);
+	g_free (s);
+
+	s = g_strdup_printf ("font_variable_size_print=%d", DEFAULT_FONT_SIZE_PRINT);
+	GET  (int, font_var_size_print, s);
+	g_free (s);
+
+	s = g_strdup_printf ("font_fixed_size_print=%d", DEFAULT_FONT_SIZE_PRINT);
+	GET  (int, font_fix_size_print, s);
+	g_free (s);
+
+	GET  (bool, font_var_points, "font_variable_points=false");
+	GET  (bool, font_fix_points, "font_fixed_points=false");
+	GET  (bool, font_var_print_points, "font_variable_print_points=false");
+	GET  (bool, font_fix_print_points, "font_fixed_print_points=false");
+
+	GET  (bool, live_spell_check, "live_spell_check=true");
+	GET  (int, spell_error_color.red,   "spell_error_color_red=65535");
+	GET  (int, spell_error_color.green, "spell_error_color_green=0");
+	GET  (int, spell_error_color.blue,  "spell_error_color_blue=0");
+	GETS (language, "language=en");
+
+	/* printf ("fonts:\n%s\n%s\n", p->font_var, p->font_fix); */
+
+	gnome_config_pop_prefix ();
+}
+
+void
+gtk_html_class_properties_save (GtkHTMLClassProperties *p)
+{
+	gnome_config_push_prefix (GTK_HTML_GNOME_CONFIG_PREFIX);
+	gnome_config_set_bool ("magic_links", p->magic_links);
+	gnome_config_set_bool ("animations", p->animations);
+	gnome_config_set_string ("keybindings_theme", p->keybindings_theme);
+	gnome_config_set_string ("font_variable", p->font_var);
+	gnome_config_set_string ("font_fixed", p->font_fix);
+	gnome_config_set_int ("font_variable_size", p->font_var_size);
+	gnome_config_set_int ("font_fixed_size", p->font_fix_size);
+	gnome_config_set_string ("font_variable_print", p->font_var_print);
+	gnome_config_set_string ("font_fixed_print", p->font_fix_print);
+	gnome_config_set_int ("font_variable_size_print", p->font_var_size_print);
+	gnome_config_set_int ("font_fixed_size_print", p->font_fix_size_print);
+	gnome_config_set_bool ("font_variable_points", p->font_var_points);
+	gnome_config_set_bool ("font_fixed_points", p->font_fix_points);
+	gnome_config_set_bool ("font_variable_print_points", p->font_var_print_points);
+	gnome_config_set_bool ("font_fixed_print_points", p->font_fix_print_points);
+
+	gnome_config_set_bool ("live_spell_check", p->live_spell_check);
+	gnome_config_set_int  ("spell_error_color_red",   p->spell_error_color.red);
+	gnome_config_set_int  ("spell_error_color_green", p->spell_error_color.green);
+	gnome_config_set_int  ("spell_error_color_blue",  p->spell_error_color.blue);
+	gnome_config_set_string ("language", p->language);
+
+	gnome_config_pop_prefix ();
+	gnome_config_sync ();
+}
+#endif
+
 static gchar *
 get_font_name (const GdkFont * font)
 {
@@ -234,92 +320,6 @@ get_default_fonts (gchar **var_name, gchar **fix_name)
 	}
 }
 
-void
-gtk_html_class_properties_load (GtkHTMLClassProperties *p)
-{
-	gchar *s, *var_name, *fix_name, *var_default, *fix_default;
-
-	get_default_fonts (&var_name, &fix_name);
-	var_default = g_strdup_printf ("font_variable=%s", var_name);
-	fix_default = g_strdup_printf ("font_fixed=%s", fix_name);
-	g_free (var_name);
-	g_free (fix_name);
-
-	gnome_config_push_prefix (GTK_HTML_GNOME_CONFIG_PREFIX);
-	GET  (bool, magic_links, "magic_links=true");
-	GET  (bool, animations, "animations=true");
-	GETS (keybindings_theme, "keybindings_theme=ms");
-	GETS (font_var, var_default);
-	GETS (font_fix, fix_default);
-	GETS (font_var_print, "font_variable_print=-*-helvetica-*-*-*-*-12-*-*-*-*-*-*-*");
-	GETS (font_fix_print, "font_fixed_print=-*-courier-*-*-*-*-12-*-*-*-*-*-*-*");
-
-	g_free (var_default);
-	g_free (fix_default);
-
-	s = g_strdup_printf ("font_variable_size=%d", DEFAULT_FONT_SIZE);
-	GET  (int, font_var_size, s);
-	g_free (s);
-
-	s = g_strdup_printf ("font_fixed_size=%d", DEFAULT_FONT_SIZE);
-	GET  (int, font_fix_size, s);
-	g_free (s);
-
-	s = g_strdup_printf ("font_variable_size_print=%d", DEFAULT_FONT_SIZE);
-	GET  (int, font_var_size_print, s);
-	g_free (s);
-
-	s = g_strdup_printf ("font_fixed_size_print=%d", DEFAULT_FONT_SIZE);
-	GET  (int, font_fix_size_print, s);
-	g_free (s);
-
-	GET  (bool, font_var_points, "font_variable_points=false");
-	GET  (bool, font_fix_points, "font_fixed_points=false");
-	GET  (bool, font_var_print_points, "font_variable_print_points=false");
-	GET  (bool, font_fix_print_points, "font_fixed_print_points=false");
-
-	GET  (bool, live_spell_check, "live_spell_check=true");
-	GET  (int, spell_error_color.red,   "spell_error_color_red=65535");
-	GET  (int, spell_error_color.green, "spell_error_color_green=0");
-	GET  (int, spell_error_color.blue,  "spell_error_color_blue=0");
-	GETS (language, "language=en");
-
-	/* printf ("fonts:\n%s\n%s\n", p->font_var, p->font_fix); */
-
-	gnome_config_pop_prefix ();
-}
-
-void
-gtk_html_class_properties_save (GtkHTMLClassProperties *p)
-{
-	gnome_config_push_prefix (GTK_HTML_GNOME_CONFIG_PREFIX);
-	gnome_config_set_bool ("magic_links", p->magic_links);
-	gnome_config_set_bool ("animations", p->animations);
-	gnome_config_set_string ("keybindings_theme", p->keybindings_theme);
-	gnome_config_set_string ("font_variable", p->font_var);
-	gnome_config_set_string ("font_fixed", p->font_fix);
-	gnome_config_set_int ("font_variable_size", p->font_var_size);
-	gnome_config_set_int ("font_fixed_size", p->font_fix_size);
-	gnome_config_set_string ("font_variable_print", p->font_var_print);
-	gnome_config_set_string ("font_fixed_print", p->font_fix_print);
-	gnome_config_set_int ("font_variable_size_print", p->font_var_size_print);
-	gnome_config_set_int ("font_fixed_size_print", p->font_fix_size_print);
-	gnome_config_set_bool ("font_variable_points", p->font_var_points);
-	gnome_config_set_bool ("font_fixed_points", p->font_fix_points);
-	gnome_config_set_bool ("font_variable_print_points", p->font_var_print_points);
-	gnome_config_set_bool ("font_fixed_print_points", p->font_fix_print_points);
-
-	gnome_config_set_bool ("live_spell_check", p->live_spell_check);
-	gnome_config_set_int  ("spell_error_color_red",   p->spell_error_color.red);
-	gnome_config_set_int  ("spell_error_color_green", p->spell_error_color.green);
-	gnome_config_set_int  ("spell_error_color_blue",  p->spell_error_color.blue);
-	gnome_config_set_string ("language", p->language);
-
-	gnome_config_pop_prefix ();
-	gnome_config_sync ();
-}
-#endif
-
 #define COPYS(v) \
         g_free (p1->v); \
         p1->v = g_strdup (p2->v);
