@@ -276,17 +276,16 @@ backward_in_flow (HTMLCursor *cursor)
 	gboolean retval;
 
 	retval = TRUE;
-	if (cursor->offset) {
-		if (html_object_is_container (cursor->object)) {
-			HTMLObject *obj;
+	if (cursor->offset  && html_object_is_container (cursor->object)) {
+		HTMLObject *obj;
 
-			obj = cursor->object;
-			while ((retval = backward (cursor)) && cursor->object != obj)
-				;
-		} else
-			retval = html_object_cursor_backward (cursor->object, cursor);
+		obj = cursor->object;
+		while ((retval = backward (cursor)) && cursor->object != obj)
+			;
 	} else {
-		if (cursor->object->prev)
+		if (cursor->offset > 1 || !cursor->object->prev)
+			retval = html_object_cursor_backward (cursor->object, cursor);
+		else if (cursor->object->prev)
 			retval = backward (cursor);
 		else
 			retval = FALSE;
@@ -648,9 +647,18 @@ html_cursor_get_position (HTMLCursor *cursor)
 }
 
 void
-html_cursor_jump_to_position (HTMLCursor *cursor,
-			      HTMLEngine *engine,
-			      gint position)
+html_cursor_jump_to_position_no_spell (HTMLCursor *cursor, HTMLEngine *engine, gint position)
+{
+	gboolean need_spell_check;
+
+	need_spell_check = engine->need_spell_check;
+	engine->need_spell_check = FALSE;
+	html_cursor_jump_to_position (cursor, engine, position);
+	engine->need_spell_check = need_spell_check;
+}
+
+void
+html_cursor_jump_to_position (HTMLCursor *cursor, HTMLEngine *engine, gint position)
 {
 	g_return_if_fail (cursor != NULL);
 	g_return_if_fail (position >= 0);
