@@ -37,6 +37,14 @@ struct _HTMLCursor {
 	gint position;
 };
 
+struct _HTMLCursorRectagle {
+	HTMLObject *object;
+	gint x1;
+	gint y1;
+	gint x2;
+	gint y2;
+};
+
 /* Lifecycle.  */
 HTMLCursor *html_cursor_new      (void);
 void        html_cursor_destroy  (HTMLCursor       *cursor);
@@ -81,10 +89,13 @@ gboolean  html_cursor_jump_to                (HTMLCursor *cursor,
 void  html_cursor_normalize     (HTMLCursor *cursor);
 
 /* Position handling.  */
-gint  html_cursor_get_position      (HTMLCursor *cursor);
-void  html_cursor_jump_to_position  (HTMLCursor *cursor,
-				     HTMLEngine *engine,
-				     gint        position);
+gint  html_cursor_get_position               (HTMLCursor *cursor);
+void  html_cursor_jump_to_position           (HTMLCursor *cursor,
+					      HTMLEngine *engine,
+					      gint        position);
+void  html_cursor_jump_to_position_no_spell  (HTMLCursor *cursor,
+					      HTMLEngine *engine,
+					      gint        position);
 
 /* Retrieving the character under/before the cursor.  */
 gunichar  html_cursor_get_current_char  (const HTMLCursor *cursor);
