@@ -304,7 +304,9 @@ html_engine_draw_cursor_in_area (HTMLEngine *engine,
 	move_rect (engine, &x1, &y1, &x2, &y2);
 	if (clip_rect (engine, x, y, width, height, &x1, &y1, &x2, &y2)) {
 		gdk_draw_line (engine->window, engine->invert_gc, x1, y1, x2, y2);
+#ifdef GTK_HTML_USE_XIM
 		gtk_html_im_position_update (engine->widget, x2, y2);
+#endif
 	}
 }
 
