@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  This file is part of the GtkHTML library.
     Copyright (C) 1999, 2000 Helix Code, Inc.
+    Copyright (C) 2001 Ximian, Inc.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -31,6 +32,7 @@
 void                       html_engine_undo                       (HTMLEngine                *e);
 void                       html_engine_redo                       (HTMLEngine                *e);
 void                       html_engine_set_mark                   (HTMLEngine                *e);
+void                       html_engine_select_spell_word_editable (HTMLEngine                *e);
 void                       html_engine_select_word_editable       (HTMLEngine                *e);
 void                       html_engine_select_line_editable       (HTMLEngine                *e);
 void                       html_engine_select_paragraph_editable  (HTMLEngine                *e);
@@ -69,13 +71,19 @@ void                       html_engine_indent_pre_line            (HTMLEngine
 void                       html_engine_fill_pre_line              (HTMLEngine                *e);
 void                       html_engine_space_and_fill_line        (HTMLEngine                *e);
 void                       html_engine_break_and_fill_line        (HTMLEngine                *e);
+gboolean                   html_engine_next_cell                  (HTMLEngine                *e,
+								   gboolean                   create);
+gboolean                   html_engine_prev_cell                  (HTMLEngine                *e);
 /*
   static (non instance) methods
 */
 gboolean                   html_is_in_word                        (gunichar                   uc);
 HTMLHAlignType             paragraph_alignment_to_html            (GtkHTMLParagraphAlignment  alignment);
-HTMLClueFlowStyle          paragraph_style_to_clueflow_style      (GtkHTMLParagraphStyle      style);
+void                       paragraph_style_to_clueflow_style      (GtkHTMLParagraphStyle      style,
+								   HTMLClueFlowStyle         *flow_style,
+								   HTMLListType              *item_type);
 GtkHTMLParagraphAlignment  html_alignment_to_paragraph            (HTMLHAlignType             alignment);
-GtkHTMLParagraphStyle      clueflow_style_to_paragraph_style      (HTMLClueFlowStyle          style);
+GtkHTMLParagraphStyle      clueflow_style_to_paragraph_style      (HTMLClueFlowStyle          style,
+								   HTMLListType               item_type);
 
 #endif /* _HTMLENGINE_EDIT_H */
