@@ -353,10 +353,12 @@ draw (HTMLObject *o,
 	GdkPixbuf *pixbuf;
 	gint base_x, base_y;
 	gint scale_width, scale_height;
-	const GdkColor *highlight_color;
+	GdkColor *highlight_color;
 	guint pixel_size;
 	ArtIRect paint;
 
+	/* printf ("Image::draw\n"); */
+
 	html_object_calc_intersection (o, &paint, x, y, width, height);
 	if (art_irect_empty (&paint))
 		return;
@@ -366,9 +368,10 @@ draw (HTMLObject *o,
 	pixbuf = image->image_ptr->pixbuf;
 	pixel_size = html_painter_get_pixel_size (painter);
 
-	if (o->selected)
-		highlight_color = &html_colorset_get_color_allocated (painter, HTMLHighlightColor)->color;
-	else
+	if (o->selected) {
+		highlight_color = &html_colorset_get_color_allocated
+			(painter, painter->focus ? HTMLHighlightColor : HTMLHighlightNFColor)->color;
+	} else
 		highlight_color = NULL;
 
 	if (pixbuf == NULL) {
@@ -670,7 +673,8 @@ html_image_init (HTMLImage *image,
 		 gboolean percent_width, gboolean percent_height,
 		 gint8 border,
 		 HTMLColor *color,
-		 HTMLVAlignType valign)
+		 HTMLVAlignType valign,
+		 gboolean reload)
 {
 	HTMLObject *object;
 
@@ -711,7 +715,7 @@ html_image_init (HTMLImage *image,
 		valign = HTML_VALIGN_BOTTOM;
 	image->valign = valign;
 
-	image->image_ptr = html_image_factory_register (imf, image, filename, FALSE);
+	image->image_ptr = html_image_factory_register (imf, image, filename, reload);
 }
 
 HTMLObject *
@@ -723,7 +727,8 @@ html_image_new (HTMLImageFactory *imf,
 		gboolean percent_width, gboolean percent_height,
 		gint8 border,	
 		HTMLColor *color,
-		HTMLVAlignType valign)
+		HTMLVAlignType valign,
+		gboolean reload)
 {
 	HTMLImage *image;
 
@@ -738,7 +743,8 @@ html_image_new (HTMLImageFactory *imf,
 			 percent_width, percent_height,
 			 border,
 			 color,
-			 valign);
+			 valign,
+			 reload);
 
 	return HTML_OBJECT (image);
 }
@@ -1032,6 +1038,7 @@ html_image_animation_timeout (HTMLImage *image)
 	HTMLEngine        *engine;
 	gint nx, ny, nex, ney;
 
+	/* printf ("animation_timeout\n"); */
 	anim->cur_frame = anim->cur_frame->next;
 	if (!anim->cur_frame)
 		anim->cur_frame = gdk_pixbuf_animation_get_frames (image->image_ptr->animation);
@@ -1064,8 +1071,8 @@ html_image_animation_timeout (HTMLImage *image)
 		}
 		
 	}
-
-	anim->timeout = g_timeout_add (10 * (gdk_pixbuf_frame_get_delay_time (frame)
+	/* printf ("timeout: %d\n", gdk_pixbuf_frame_get_delay_time (frame)); */
+	anim->timeout = g_timeout_add (10 * (gdk_pixbuf_frame_get_delay_time (frame) > 0
 					     ? gdk_pixbuf_frame_get_delay_time (frame) : 1),
 				       (GtkFunction) html_image_animation_timeout, (gpointer) image);
 
@@ -1103,6 +1110,8 @@ html_image_animation_stop (HTMLImageAnimation *anim)
 static void
 html_image_factory_frame_done (GdkPixbufLoader *loader, GdkPixbufFrame *frame, HTMLImagePointer *ip)
 {
+	/* printf ("frame done\n"); */
+
 	if (!ip->animation) {
 		ip->animation = gdk_pixbuf_loader_get_animation (loader);
 		gdk_pixbuf_animation_ref (ip->animation);
