@@ -26,6 +26,7 @@
 
 #include "htmlclue.h"
 #include "htmlclueflow.h"
+#include "htmlcluealigned.h"
 #include "htmlcluev.h"
 #include "htmlcolor.h"
 #include "htmlcolorset.h"
@@ -58,19 +59,21 @@ free_data (GQuark id, gpointer data, gpointer user_data)
 static void
 destroy (HTMLObject *self)
 {
+#define GTKHTML_MEM_DEBUG 1
+#if GTKHTML_MEM_DEBUG
+	self->parent = HTML_OBJECT (0xdeadbeef);
+	self->next = HTML_OBJECT (0xdeadbeef);
+	self->prev = HTML_OBJECT (0xdeadbeef);
+#else
+	self->next = NULL;
+	self->prev = NULL;
+#endif
+	g_datalist_foreach (&self->object_data, free_data, NULL);
+	g_datalist_clear (&self->object_data);
+	
 	if (self->redraw_pending) {
 		self->free_pending = TRUE;
 	} else {
-
-#if GTKHTML_MEM_DEBUG
-		self->next = 0xdeadbeef;
-		self->prev = 0xdeadbeef;
-#else
-		self->next = NULL;
-		self->prev = NULL;
-#endif
-		g_datalist_foreach (&self->object_data, free_data, NULL);
-		g_datalist_clear (&self->object_data);
 		g_free (self);
 	}
 }
@@ -93,7 +96,7 @@ copy (HTMLObject *self,
 	dest->pref_width = self->pref_width;
 	dest->percent = self->percent;
 	dest->flags = self->flags;
-	dest->redraw_pending = self->redraw_pending;
+	dest->redraw_pending = FALSE;
 	dest->selected = self->selected;
 	dest->free_pending = FALSE;
 	dest->change = self->change;
@@ -140,7 +143,7 @@ op_cut (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GList *left, GL
 }
 
 static gboolean
-merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList *left, GList *right)
+merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList **left, GList **right, HTMLCursor *cursor)
 {
 	return FALSE;
 }
@@ -163,17 +166,19 @@ split (HTMLObject *self, HTMLEngine *e, HTMLObject *child, gint offset, gint lev
 	}
 
 	if (offset) {
-		if (!self->next)
+		if (!self->next) {
 			html_clue_append (HTML_CLUE (self->parent), html_engine_new_text_empty (e));
+		}
 		*left  = g_list_prepend (*left,  self);
 		*right = g_list_prepend (*right, self->next);
 	} else {
-		if (!self->prev)
-			html_clue_prepend (HTML_CLUE (self->parent), html_engine_new_text_empty (e));
+		if (!self->prev) {
+			e->cursor->object = html_engine_new_text_empty (e);
+			e->cursor->offset = 0;
+			html_clue_prepend (HTML_CLUE (self->parent), e->cursor->object);
+		}
 		*left  = g_list_prepend (*left,  self->prev);
 		*right = g_list_prepend (*right, self);
-		e->cursor->object = self->prev;
-		e->cursor->offset = html_object_get_length (self->prev);
 	}
 	level--;
 
@@ -225,30 +230,27 @@ fit_line (HTMLObject *o,
 	  gboolean first_run,
 	  gint width_left)
 {
-	html_object_calc_size (o, painter);
+	html_object_calc_size (o, painter, FALSE);
 	return (o->width <= width_left || first_run) ? HTML_FIT_COMPLETE : HTML_FIT_NONE;
 }
 
 static gboolean
-calc_size (HTMLObject *o,
-	   HTMLPainter *painter)
+calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 {
 	return FALSE;
 }
 
 static gint
-calc_min_width (HTMLObject *o,
-		HTMLPainter *painter)
+calc_min_width (HTMLObject *o, HTMLPainter *painter)
 {
-	html_object_calc_size (o, painter);
+	html_object_calc_size (o, painter, FALSE);
 	return o->width;
 }
 
 static gint
-calc_preferred_width (HTMLObject *o,
-		      HTMLPainter *painter)
+calc_preferred_width (HTMLObject *o, HTMLPainter *painter)
 {
-	html_object_calc_size (o, painter);
+	html_object_calc_size (o, painter, FALSE);
 	return o->width;
 }
 
@@ -288,9 +290,9 @@ set_painter (HTMLObject *o, HTMLPainter *painter)
 static void
 reset (HTMLObject *o)
 {
-	o->width = 0;
-	o->ascent = 0;
-	o->descent = 0;
+	/* o->width = 0;
+	   o->ascent = 0;
+	   o->descent = 0; */
 }
 
 static const gchar *
@@ -385,7 +387,7 @@ relayout (HTMLObject *self,
 		self->y -= prev_ascent + prev_descent;
 	}
 
-	changed = html_object_calc_size (self, engine->painter);
+	changed = html_object_calc_size (self, engine->painter, FALSE);
 
 	if (prev_width == self->width
 	    && prev_ascent == self->ascent
@@ -742,13 +744,13 @@ html_object_op_cut (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GLi
 }
 
 gboolean
-html_object_merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList *left, GList *right)
+html_object_merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList **left, GList **right, HTMLCursor *cursor)
 {
 	if ((HTML_OBJECT_TYPE (self) == HTML_OBJECT_TYPE (with)
 	     /* FIXME */
 	     || (HTML_OBJECT_TYPE (self) == HTML_TYPE_TABLECELL && HTML_OBJECT_TYPE (with) == HTML_TYPE_CLUEV)
 	     || (HTML_OBJECT_TYPE (with) == HTML_TYPE_TABLECELL && HTML_OBJECT_TYPE (self) == HTML_TYPE_CLUEV))
-	    && (* HO_CLASS (self)->merge) (self, with, e, left, right)) {
+	    && (* HO_CLASS (self)->merge) (self, with, e, left, right, cursor)) {
 		if (with->parent)
 			html_object_remove_child (with->parent, with);
 		html_object_destroy (with);
@@ -873,10 +875,14 @@ html_object_fit_line (HTMLObject *o,
 }
 
 gboolean
-html_object_calc_size (HTMLObject *o,
-		       HTMLPainter *painter)
+html_object_calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 {
-	return (* HO_CLASS (o)->calc_size) (o, painter);
+	gboolean rv;
+
+	rv = (* HO_CLASS (o)->calc_size) (o, painter, changed_objs);
+	o->change &= ~HTML_CHANGE_SIZE;
+
+	return rv;
 }
 
 void
@@ -949,10 +955,13 @@ html_object_calc_preferred_width (HTMLObject *o,
 	return o->pref_width;
 }
 
+#if 0
 gint
-html_object_get_uris (HTMLObject *o, char **link, char **target, char **src) {
+html_object_get_uris (HTMLObject *o, char **link, char **target, char **src) 
+{
 	return TRUE;
 }
+#endif 
 
 const gchar *
 html_object_get_url (HTMLObject *o)
@@ -1038,6 +1047,12 @@ html_object_get_cursor (HTMLObject *self,
 			gint *x2, gint *y2)
 {
 	(* HO_CLASS (self)->get_cursor) (self, painter, offset, x1, y1, x2, y2);
+	if (!html_object_is_text (self) && *y2 - *y1 < 10) {
+		gint missing = 10 - (*y2 - *y1);
+
+		*y1 -= (missing >> 1) + ((missing >> 1) & 1);
+		*y2 += missing >> 1;
+	}
 }
 
 /* Warning: `calc_size()' must have been called on `self' before this so that
@@ -1110,6 +1125,19 @@ html_object_is_text (HTMLObject *object)
 	return (type == HTML_TYPE_TEXT || type == HTML_TYPE_LINKTEXT);
 }
 
+gboolean
+html_object_is_clue (HTMLObject *object)
+{
+	HTMLType type;
+
+	g_return_val_if_fail (object != NULL, FALSE);
+
+	type = HTML_OBJECT_TYPE (object);
+
+	return (type == HTML_TYPE_CLUE || type == HTML_TYPE_CLUEV || type == HTML_TYPE_TABLECELL
+		|| type == HTML_TYPE_CLUEFLOW || type == HTML_TYPE_CLUEALIGNED);
+}
+
 HTMLObject *
 html_object_next_not_type (HTMLObject *object, HTMLType t)
 {
@@ -1337,13 +1365,16 @@ html_object_cursor_forward (HTMLObject *self, HTMLCursor *cursor)
 gboolean
 html_object_cursor_backward (HTMLObject *self, HTMLCursor *cursor)
 {
+	HTMLObject *prev;
+
 	g_assert (self);
 	g_assert (cursor->object == self);
 
 	if (html_object_is_container (self))
 		return FALSE;
 
-	if (cursor->offset > 1 || (!html_object_prev_not_slave (self) && cursor->offset > 0)) {
+	if (cursor->offset > 1 || (cursor->offset > 0 && (! (prev = html_object_prev_not_slave (self))
+							  || HTML_IS_CLUEALIGNED (prev)))) {
 		cursor->offset --;
 		cursor->position --;
 		return TRUE;
@@ -1459,27 +1490,30 @@ static HTMLObject *
 move_object_cursor (HTMLObject *obj, gint *offset, gboolean forward,
 		    HTMLObject * (*next_fn ) (HTMLObject *), HTMLObject * (*down_fn ) (HTMLObject *))
 {
-	HTMLObject *down;
+	HTMLObject *down, *before;
 
 	if (((*offset == 0 && forward) || (*offset && !forward)) && html_object_is_container (obj))
 		if ((down = (*down_fn) (obj))) {
 			down = move_object_downtree_cursor (down, down_fn);
 			if (down) {
 				if (html_object_is_container (down))
-					*offset = forward ? 0 : 1; /* FIXME for prev it's reversed */
+					*offset = forward ? 0 : 1;
 				return down;
 			}
 		}
 
+	before = obj;
 	obj = next_object_uptree_cursor (obj, next_fn);
 	if (obj) {
 		if (html_object_accepts_cursor (obj)) {
 			if (html_object_is_container (obj))
-				*offset = forward ? 1 : 0; /* FIXME for prev it's reversed */
+				*offset = before->parent == obj->parent
+					? forward ? 0 : 1
+					: forward ? 1 : 0;
 		} else {
 			obj = move_object_downtree_cursor (obj, down_fn);
 			if (html_object_is_container (obj))
-				*offset = forward ? 0 : 1; /* FIXME for prev it's reversed */
+				*offset = forward ? 0 : 1;
 		}
 	}
 
@@ -1606,7 +1640,7 @@ unselect_object (HTMLObject *o, HTMLEngine *e, gpointer data)
 }
 
 gchar *
-html_object_get_selection_string (HTMLObject *o)
+html_object_get_selection_string (HTMLObject *o, HTMLEngine *e)
 {
 	HTMLObject *tail;
 	tmpSelData data;
@@ -1619,9 +1653,9 @@ html_object_get_selection_string (HTMLObject *o)
 	data.in     = FALSE;
 	data.i      = html_interval_new (html_object_get_head_leaf (o), tail, 0, html_object_get_length (tail));
 
-	html_interval_forall (data.i, NULL, select_object, &data);
+	html_interval_forall (data.i, e, select_object, &data);
 	html_object_append_selection_string (o, data.buffer);
-	html_interval_forall (data.i, NULL, unselect_object, NULL);
+	html_interval_forall (data.i, e, unselect_object, NULL);
 
 	html_interval_destroy (data.i);
 	string = data.buffer->str;
@@ -1736,7 +1770,7 @@ merge_down (HTMLEngine *e, GList *left, GList *right)
 		ro    = HTML_OBJECT (right->data);
 		left  = left->next;
 		right = right->next;
-		if (!html_object_merge (lo, ro, e, left, right))
+		if (!html_object_merge (lo, ro, e, &left, &right, NULL))
 			break;
 	}
 }
@@ -1746,3 +1780,85 @@ html_object_merge_down (HTMLObject *o, HTMLObject *w, HTMLEngine *e)
 {
 	merge_down (e, html_object_tails_list (o), html_object_heads_list (w));
 }
+
+gboolean
+html_object_is_parent (HTMLObject *parent, HTMLObject *child)
+{
+	g_assert (parent && child);
+
+	while (child) {
+		if (child->parent == parent)
+			return TRUE;
+		child = child->parent;
+	}
+
+	return FALSE;
+}
+
+gint
+html_object_get_insert_level (HTMLObject *o)
+{
+	switch (HTML_OBJECT_TYPE (o)) {
+	case HTML_TYPE_TABLECELL:
+	case HTML_TYPE_CLUEV:
+		return 3;
+	case HTML_TYPE_CLUEFLOW:
+		return 2;
+	default:
+		return 1;
+	}
+}
+
+void
+html_object_engine_translation (HTMLObject *o, HTMLEngine *e, gint *tx, gint *ty)
+{
+	HTMLObject *p;
+
+	*tx = 0;
+	*ty = 0;
+
+	for (p = o->parent; p != NULL && HTML_OBJECT_TYPE (p) != HTML_TYPE_IFRAME; p = p->parent) {
+		*tx += p->x;
+		*ty += p->y - p->ascent;
+	}
+
+	*tx = *tx + e->leftBorder - e->x_offset;
+	*ty = *ty + e->topBorder - e->y_offset;
+
+}
+
+gboolean
+html_object_engine_intersection (HTMLObject *o, HTMLEngine *e, gint tx, gint ty, gint *x1, gint *y1, gint *x2, gint *y2)
+{
+	*x1 = o->x + tx;
+	*y1 = o->y - o->ascent + ty;
+	*x2 = o->x + o->width + tx;
+	*y2 = o->y + o->descent + ty;
+
+	return html_engine_intersection (e, x1, y1, x2, y2);
+}
+
+void
+html_object_add_to_changed (GList **changed_objs, HTMLObject *o)
+{
+	GList *l, *next;
+
+	if (!changed_objs || (*changed_objs && (*changed_objs)->data == o))
+		return;
+
+	for (l = *changed_objs; l; l = next) {
+		if (l->data == NULL) {
+			l = l->next;
+			next = l->next;
+			continue;
+		}
+		next = l->next;
+		if (html_object_is_parent (o, HTML_OBJECT (l->data))) {
+			*changed_objs = g_list_remove_link (*changed_objs, l);
+			g_list_free (l);
+		} else
+			break;
+	}
+
+	*changed_objs = g_list_prepend (*changed_objs, o);
+}
