@@ -70,6 +70,14 @@ struct _HTMLObject {
 	GData *object_data;
 };
 
+struct _HTMLObjectClearRectangle {
+	HTMLObject *object;
+	gint x;
+	gint y;
+	gint width;
+	gint height;
+};
+
 struct _HTMLObjectClass {
 	HTMLType type;
 
@@ -98,8 +106,9 @@ struct _HTMLObjectClass {
 	gboolean     (* merge)           (HTMLObject *self,
 					  HTMLObject *o,
 					  HTMLEngine *e,
-					  GList      *left,
-					  GList      *right);
+					  GList      **left,
+					  GList      **right,
+					  HTMLCursor *cursor);
 	void         (* remove_child)    (HTMLObject *self,
 					  HTMLObject *child);
 	void         (* split)           (HTMLObject *self,
@@ -115,7 +124,7 @@ struct _HTMLObjectClass {
 	HTMLFitType (* fit_line) (HTMLObject *o, HTMLPainter *painter,
 				  gboolean start_of_line, gboolean first_run,
 				  gint width_left);
-	gboolean (* calc_size) (HTMLObject *o, HTMLPainter *painter);
+	gboolean (* calc_size) (HTMLObject *o, HTMLPainter *painter, GList **changed_objs);
 	gint (* calc_min_width) (HTMLObject *o, HTMLPainter *painter);
 	gint (* calc_preferred_width) (HTMLObject *o, HTMLPainter *painter);
 	void (* set_max_ascent) (HTMLObject *o, HTMLPainter *painter, gint a);
@@ -274,8 +283,9 @@ HTMLObject     *html_object_op_cut                (HTMLObject            *self,
 gboolean        html_object_merge                 (HTMLObject            *self,
 						   HTMLObject            *with,
 						   HTMLEngine            *e,
-						   GList                 *left,
-						   GList                 *right);
+						   GList                 **left,
+						   GList                 **right,
+						   HTMLCursor            *cursor);
 void            html_object_remove_child          (HTMLObject            *self,
 						   HTMLObject            *child);
 void            html_object_split                 (HTMLObject            *self,
@@ -298,6 +308,7 @@ void            html_object_set_painter           (HTMLObject            *o,
 void            html_object_clear_word_width      (HTMLObject            *o);
 void            html_object_reset                 (HTMLObject            *o);
 gboolean        html_object_is_text               (HTMLObject            *object);
+gboolean        html_object_is_clue               (HTMLObject            *object);
 HTMLEngine     *html_object_get_engine            (HTMLObject            *self,
 						   HTMLEngine            *e);
 void            html_object_forall                (HTMLObject            *self,
@@ -354,7 +365,8 @@ HTMLFitType     html_object_fit_line              (HTMLObject            *o,
 						   gboolean               first_run,
 						   gint                   width_left);
 gboolean        html_object_calc_size             (HTMLObject            *o,
-						   HTMLPainter           *painter);
+						   HTMLPainter           *painter,
+						   GList                **changed_objs);
 void            html_object_set_max_ascent        (HTMLObject            *o,
 						   HTMLPainter           *painter,
 						   gint                   a);
@@ -469,7 +481,8 @@ gboolean    html_object_select_range             (HTMLObject *self,
 						  gboolean    queue_draw);
 void        html_object_append_selection_string  (HTMLObject *self,
 						  GString    *buffer);
-gchar      *html_object_get_selection_string     (HTMLObject *o);
+gchar      *html_object_get_selection_string     (HTMLObject *o,
+						  HTMLEngine *e);
 
 /* Saving.  */
 gboolean  html_object_save  (HTMLObject          *self,
@@ -517,5 +530,24 @@ GList    *html_object_tails_list                 (HTMLObject *o);
 void      html_object_merge_down                 (HTMLObject *o,
 						  HTMLObject *w,
 						  HTMLEngine *e);
+gboolean  html_object_is_parent                  (HTMLObject *parent,
+						  HTMLObject *child);
+gint      html_object_get_insert_level           (HTMLObject *o);
+
+void      html_object_engine_translation   (HTMLObject *o,
+					    HTMLEngine *e,
+					    gint       *tx,
+					    gint       *ty);
+gboolean  html_object_engine_intersection  (HTMLObject *o,
+					    HTMLEngine *e,
+					    gint        tx,
+					    gint        ty,
+					    gint       *x1,
+					    gint       *y1,
+					    gint       *x2,
+					    gint       *y2);
+
+void  html_object_add_to_changed  (GList      **changed_objs,
+				   HTMLObject  *o);
 
 #endif /* _HTMLOBJECT_H_ */
