@@ -21,15 +21,30 @@
 
 #include <gtk/gtkselection.h>
 #include "htmlcursor.h"
+#include "htmlengine-edit-cursor.h"
+#include "htmlentity.h"
 #include "htmlinterval.h"
 #include "htmlselection.h"
 #include "htmlengine-edit.h"
 #include "htmlengine-edit-selection-updater.h"
 
+guint32
+html_selection_current_time (void)
+{
+	GdkEvent *event;
+
+	event = gtk_get_current_event ();
+	if (event != NULL)
+		return gdk_event_get_time (event);
+
+	return GDK_CURRENT_TIME;
+}
+
 void
 html_engine_select_interval (HTMLEngine *e, HTMLInterval *i)
 {
 	e = html_engine_get_top_html_engine (e);
+	html_engine_hide_cursor (e);
 	if (e->selection && html_interval_eq (e->selection, i))
 		html_interval_destroy (i);
 	else {
@@ -37,7 +52,9 @@ html_engine_select_interval (HTMLEngine *e, HTMLInterval *i)
 		e->selection = i;
 		html_interval_select (e->selection, e);
 	}
-	html_engine_activate_selection (e, GDK_CURRENT_TIME);
+
+	html_engine_activate_selection (e, html_selection_current_time ());
+	html_engine_show_cursor (e);
 }
 
 void
@@ -78,6 +95,11 @@ html_engine_clear_selection (HTMLEngine *e)
 		html_interval_destroy (e->selection);
 		html_engine_edit_selection_updater_reset (e->selection_updater);
 		e->selection = NULL;
+		/*
+		if (gdk_selection_owner_get (GDK_SELECTION_PRIMARY) == GTK_WIDGET (e->widget)->window)
+			gtk_selection_owner_set (NULL, GDK_SELECTION_PRIMARY, 
+						 html_selection_current_time ());    
+		*/
 	}
 }
 
@@ -86,8 +108,10 @@ html_engine_unselect_all (HTMLEngine *e)
 {
 	e = html_engine_get_top_html_engine (e);
 	if (e->selection) {
+		html_engine_hide_cursor (e);
 		html_interval_unselect (e->selection, e);
 		html_engine_clear_selection (e);
+		html_engine_show_cursor (e);
 	}
 }
 
@@ -116,9 +140,11 @@ html_engine_disable_selection (HTMLEngine *e)
 	g_return_if_fail (e != NULL);
 	g_return_if_fail (HTML_IS_ENGINE (e));
 
+	html_engine_hide_cursor (e);
 	remove_mark (e);
 	html_engine_unselect_all (e);
 	e->selection_mode = FALSE;
+	html_engine_show_cursor (e);
 }
 
 static gboolean
@@ -127,14 +153,32 @@ line_interval (HTMLEngine *e, HTMLCursor *begin, HTMLCursor *end)
 	return html_cursor_beginning_of_line (begin, e) && html_cursor_end_of_line (end, e);
 }
 
+gboolean
+html_selection_word (gunichar uc)
+{
+	return uc && uc != ' ' && uc != '\t' && uc != ENTITY_NBSP         /* white space */
+		&& uc != '(' && uc != ')' && uc != '[' && uc != ']';
+}
+
+gboolean
+html_selection_spell_word (gunichar uc, gboolean *cited)
+{
+	if (uc == '\'' || uc == '`') {
+		*cited = TRUE;
+		return FALSE;
+	} else {
+		return g_unichar_isalpha (uc);
+	}
+}
+
 static gboolean
 word_interval (HTMLEngine *e, HTMLCursor *begin, HTMLCursor *end)
 {
 	/* move to the begin of word */
-	while (html_is_in_word (html_cursor_get_prev_char (begin)))
+	while (html_selection_word (html_cursor_get_prev_char (begin)))
 		html_cursor_backward (begin, e);
 	/* move to the end of word */
-	while (html_is_in_word (html_cursor_get_current_char (end)))
+	while (html_selection_word (html_cursor_get_current_char (end)))
 		html_cursor_forward (end, e);
 
 	return (begin->object && end->object);
