@@ -94,22 +94,27 @@ get_tags (const HTMLText *text,
 	size = font_style & GTK_HTML_FONT_STYLE_SIZE_MASK;
 	if (size != 0) {
 		opening_p += sprintf (opening_p, "<FONT SIZE=\"%d\">", size);
+		ending_p += sprintf (ending_p, "</FONT>");
 	}
 
 	if (font_style & GTK_HTML_FONT_STYLE_BOLD) {
 		opening_p += sprintf (opening_p, "<B>");
+		ending_p += sprintf (ending_p, "</B>");
 	}
 
 	if (font_style & GTK_HTML_FONT_STYLE_ITALIC) {
 		opening_p += sprintf (opening_p, "<I>");
+		ending_p += sprintf (ending_p, "</I>");
 	}
 
 	if (font_style & GTK_HTML_FONT_STYLE_UNDERLINE) {
 		opening_p += sprintf (opening_p, "<U>");
+		ending_p += sprintf (ending_p, "</U>");
 	}
 
 	if (font_style & GTK_HTML_FONT_STYLE_STRIKEOUT) {
 		opening_p += sprintf (opening_p, "<S>");
+		ending_p += sprintf (ending_p, "</S>");
 	}
 
 	if (font_style & GTK_HTML_FONT_STYLE_FIXED) {
@@ -117,27 +122,6 @@ get_tags (const HTMLText *text,
 		ending_p += sprintf (ending_p, "</TT>");
 	}
 
-	if (font_style & GTK_HTML_FONT_STYLE_STRIKEOUT) {
-		ending_p += sprintf (ending_p, "</S>");
-	}
-
-	if (font_style & GTK_HTML_FONT_STYLE_UNDERLINE) {
-		ending_p += sprintf (ending_p, "</U>");
-	}
-
-	if (font_style & GTK_HTML_FONT_STYLE_ITALIC) {
-		ending_p += sprintf (ending_p, "</I>");
-	}
-
-	if (font_style & GTK_HTML_FONT_STYLE_BOLD) {
-		ending_p += sprintf (ending_p, "</B>");
-	}
-
-	if (size != 0) {
-		ending_p += sprintf (ending_p, "</FONT SIZE=\"%d\">", size);
-	}
-
-
 	*opening_p = 0;
 	*ending_p = 0;
 }
@@ -220,95 +204,6 @@ html_text_clear_word_width (HTMLText *text)
 	text->words = 0;
 }
 
-static void
-merge_word_width (HTMLText *t1, HTMLText *t2, HTMLPainter *p)
-{
-	guint len, i, words;
-
-	/* printf ("before merge '%s' '%s'\n", t1->text, t2->text);
-	   debug_word_width (t1);
-	   debug_word_width (t2); */
-
-	if (!t1->word_width)
-		return;
-
-	len = strlen (t1->text);
-	if (((len && t1->text [len - 1] == ' ')
-	     || (len > 1 && (guchar) t1->text [len - 1] == 0xa0 && (guchar) t1->text [len - 1] == 0xc2))
-	    && t2->text [0] == ' ') {
-		html_text_clear_word_width (t1);
-		return; /* we don't want do merge as convert_nbsp will 100% happen */
-	}
-
-	/* temporarily disable word width merging because of tabs */
-	html_text_clear_word_width (t1);
-	return;
-
-	if (!t2->word_width)
-		html_text_request_word_width (t2, p);
-
-	words          = t1->words;
-	t1->words      = words + t2->words - 1;
-	t1->word_width = g_renew (guint, t1->word_width, t1->words);
-
-	for (i = 1; i < t2->words; i ++)
-		t1->word_width [words + i - 1] = t2->word_width [i] + t1->word_width [words - 1];
-	t1->word_width [words - 1] += t2->word_width [0];
-
-	/* printf ("after merge '%s%s'\n", t1->text, t2->text);
-	   debug_word_width (t1); */
-}
-
-static void
-split_word_width (HTMLText *s, HTMLText *d, HTMLPainter *p, gint offset)
-{
-	gchar *str;
-	guint words, i;
-	gboolean in_middle;
-
-	html_text_clear_word_width (d);
-	if (!s->word_width)
-		return;
-
-	/* printf ("before split '%s%s'\n", s->text, d->text);
-	   debug_word_width (s);
-	   debug_word_width (d); */
-
-	words     = get_words (s->text);
-	in_middle = d->text [0] == ' ' ? FALSE : TRUE;
-
-	/* fill d */
-	d->words      = s->words - words + 1;
-	d->word_width = g_new (guint, d->words);
-	if (in_middle) {
-		str = strchr (d->text, ' ');
-		d->word_width [0] = html_painter_calc_text_width (p, d->text, str
-								  ? g_utf8_pointer_to_offset (d->text, str)
-								  : d->text_len, -1, html_text_get_font_style (s), s->face);
-	} else
-		d->word_width [0] = 0;
-	for (i = 1; i < d->words; i ++)
-		d->word_width [i] = s->word_width [words + i - 1] - s->word_width [words - 1] + d->word_width [0];
-
-	/* fill s */
-	if (s->words != words) {
-		s->words = words;
-		s->word_width = g_renew (guint, s->word_width, s->words);
-	}
-
-	if (in_middle) {
-		str = strrchr (s->text, ' ');
-		if (!str)
-			str = s->text;
-		s->word_width [s->words - 1] = html_painter_calc_text_width (p, str, g_utf8_strlen (str, -1), -1,
-									     html_text_get_font_style (s), s->face)
-			+ (s->words > 1 ? s->word_width [s->words - 2] : 0);
-	}
-	/* printf ("after split '%s' '%s'\n", s->text, d->text);
-	   debug_word_width (s);
-	   debug_word_width (d); */
-}
-
 HTMLObject *
 html_text_op_copy_helper (HTMLText *text, GList *from, GList *to, guint *len, HTMLTextHelperFunc f)
 {
@@ -371,6 +266,9 @@ html_text_op_cut_helper (HTMLText *text, HTMLEngine *e, GList *from, GList *to,
 	} else {
 		text->spell_errors = remove_spell_errors (text->spell_errors, 0, text->text_len);
 		html_object_move_cursor_before_remove (HTML_OBJECT (text), e);
+		html_object_change_set (HTML_OBJECT (text)->parent, HTML_CHANGE_ALL_CALC);
+		/* force parent redraw */
+		HTML_OBJECT (text)->parent->width = 0;
 		html_object_remove_child (HTML_OBJECT (text)->parent, HTML_OBJECT (text));
 
 		rv    = HTML_OBJECT (text);
@@ -406,7 +304,7 @@ op_cut (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GList *left, GL
 }
 
 static gboolean
-object_merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList *left, GList *right)
+object_merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList **left, GList **right, HTMLCursor *cursor)
 {
 	HTMLText *t1, *t2;
 	gchar *to_free;
@@ -419,7 +317,12 @@ object_merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList *left, GL
 
 	/* printf ("merge '%s' '%s'\n", t1->text, t2->text); */
 
-	merge_word_width (t1, t2, e->painter);
+	/* merge_word_width (t1, t2, e->painter); */
+
+	if (e->cursor->object == with) {
+		e->cursor->object  = self;
+		e->cursor->offset += t1->text_len;
+	}
 
 	move_spell_errors (t2->spell_errors, 0, t1->text_len);
 	t1->spell_errors = g_list_concat (t1->spell_errors, t2->spell_errors);
@@ -432,7 +335,8 @@ object_merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList *left, GL
 	html_text_convert_nbsp (t1, TRUE);
 	html_object_change_set (self, HTML_CHANGE_ALL_CALC);
 
-	html_text_request_word_width (t1, e->painter);
+	html_text_clear_word_width (t1);
+	/* html_text_request_word_width (t1, e->painter); */
 	/* printf ("merged '%s'\n", t1->text); */
 	/* printf ("--- after merge\n");
 	   debug_spell_errors (t1->spell_errors);
@@ -466,24 +370,16 @@ object_split (HTMLObject *self, HTMLEngine *e, HTMLObject *child, gint offset, g
 	t2->text_len   -= offset;
 	if (!html_text_convert_nbsp (t2, FALSE))
 		t2->text = g_strdup (t2->text);
-	else {  /* take care of split in &nbsp; sequence */
-		guint len = strlen (t1->text);
-
-		if (t2->text [0] == ' '
-		    && ((len > 1 && (guchar) t1->text [len - 1] == 0xa0 && (guchar) t2->text [len - 2] == 0xc2)
-			|| (len && t1->text [len - 1] == ' ')))
-			html_text_clear_word_width (t1);
-	}
 	g_free (tt);
 
 	html_clue_append_after (HTML_CLUE (self->parent), dup, self);
 
 	prev = self->prev;
-	if (t1->text_len == 0 && prev && html_object_merge (prev, self, e, NULL, NULL))
+	if (t1->text_len == 0 && prev && html_object_merge (prev, self, e, NULL, NULL, NULL))
 		self = prev;
 
 	if (t2->text_len == 0 && dup->next)
-		html_object_merge (dup, dup->next, e, NULL, NULL);
+		html_object_merge (dup, dup->next, e, NULL, NULL, NULL);
 
 	/* printf ("--- before split offset %d dup len %d\n", offset, HTML_TEXT (dup)->text_len);
 	   debug_spell_errors (HTML_TEXT (self)->spell_errors); */
@@ -507,9 +403,8 @@ object_split (HTMLObject *self, HTMLEngine *e, HTMLObject *child, gint offset, g
 	html_object_change_set (self, HTML_CHANGE_ALL_CALC);
 	html_object_change_set (dup,  HTML_CHANGE_ALL_CALC);
 
-	split_word_width (HTML_TEXT (self), HTML_TEXT (dup), e->painter, offset);
-	/* html_text_clear_word_width (HTML_TEXT (self));
-	   html_text_clear_word_width (HTML_TEXT (dup)); */
+	html_text_clear_word_width (HTML_TEXT (self));
+	html_text_clear_word_width (HTML_TEXT (dup));
 
 	level--;
 	if (level)
@@ -517,8 +412,7 @@ object_split (HTMLObject *self, HTMLEngine *e, HTMLObject *child, gint offset, g
 }
 
 static gboolean
-calc_size (HTMLObject *self,
-	   HTMLPainter *painter)
+calc_size (HTMLObject *self, HTMLPainter *painter, GList **changed_objs)
 {
 	HTMLText *text = HTML_TEXT (self);
 	GtkHTMLFontStyle style = html_text_get_font_style (text);
@@ -606,7 +500,7 @@ calc_word_width (HTMLText *text, HTMLPainter *painter, gint line_offset)
 			+ html_painter_calc_text_width (painter,
 							begin, end ? g_utf8_pointer_to_offset (begin, end)
 							: g_utf8_strlen (begin, -1),
-							line_offset, font_style, text->face);
+							&line_offset, font_style, text->face);
 		begin = end;
 	}
 
@@ -683,7 +577,7 @@ forward_get_nb_width (HTMLText *text, HTMLPainter *painter, gboolean begin)
 	g_assert (text->text_len == 0);
 
 	/* find prev/next object */
-	obj = (begin)
+	obj = begin
 		? html_object_prev_not_slave (HTML_OBJECT (text))
 		: html_object_next_not_slave (HTML_OBJECT (text));
 
@@ -694,6 +588,27 @@ forward_get_nb_width (HTMLText *text, HTMLPainter *painter, gboolean begin)
 		return html_text_get_nb_width (HTML_TEXT (obj), painter, begin);
 }
 
+static gint
+get_next_nb_width (HTMLText *text, HTMLPainter *painter, gboolean begin)
+{
+	HTMLObject *obj;
+
+	g_assert (text);
+	g_assert (html_object_is_text (HTML_OBJECT (text)));
+	g_assert (text->words == 1);
+
+	/* find prev/next object */
+	obj = begin
+		? html_object_next_not_slave (HTML_OBJECT (text))
+		: html_object_prev_not_slave (HTML_OBJECT (text));
+
+	/* if not found or not text return 0, otherwise forward get_nb_with there */
+	if (!obj || !html_object_is_text (obj))
+		return 0;
+	else
+		return html_text_get_nb_width (HTML_TEXT (obj), painter, begin);
+}
+
 static inline guint
 word_width (HTMLText *text, HTMLPainter *p, guint i)
 {
@@ -719,7 +634,8 @@ html_text_get_nb_width (HTMLText *text, HTMLPainter *painter, gboolean begin)
 
 	html_text_request_word_width (text, painter);
 
-	return word_width (text, painter, begin ? 0 : text->words - 1);
+	return word_width (text, painter, begin ? 0 : text->words - 1)
+		+ (text->words == 1 ? get_next_nb_width (text, painter, begin) : 0);
 }
 
 static gint
@@ -817,70 +733,99 @@ get_length (HTMLObject *self)
 
 /* #define DEBUG_NBSP */
 
-static gboolean
-is_convert_nbsp_needed (const gchar *s, gint *delta_out)
+static inline gboolean
+check_last_white (gboolean rv, gint white_space, gunichar last_white, gint *delta_out)
 {
-	gunichar uc;
-	gboolean rv = FALSE;
-	gboolean in_white_space;
-	const gchar *p, *op;
-
-	*delta_out = 0;
-
-	op = p = s;
-	in_white_space = FALSE;
-	while (*p && (p = e_unicode_get_utf8 (p, &uc))) {
-		if (uc == ENTITY_NBSP) {
-			if (!in_white_space) {
-				(*delta_out) --;
-				rv = TRUE;
-			}
-			in_white_space = TRUE;
-		} else if (uc == ' ') {
-			if (in_white_space) {
-				(*delta_out) ++;
-				rv = TRUE;
-			}
-			in_white_space = TRUE;
-		} else
-			in_white_space = FALSE;
-
-		op = p;
+	if (white_space > 0 && last_white == ENTITY_NBSP) {
+		(*delta_out) --;
+		rv = TRUE;
 	}
 
 	return rv;
 }
 
+static inline gboolean
+check_prev_white (gboolean rv, gint white_space, gunichar last_white, gint *delta_out)
+{
+	if (white_space > 0 && last_white == ' ') {
+		(*delta_out) ++;
+		rv = TRUE;
+	}
+
+	return rv;
+}
+
+static gboolean
+is_convert_nbsp_needed (const gchar *s, gint *delta_out)
+{
+	gunichar uc, last_white = 0;
+	gboolean rv = FALSE;
+	gint white_space;
+	const gchar *p, *op;
+
+	*delta_out = 0;
+
+	op = p = s;
+	white_space = 0;
+	while (*p && (p = e_unicode_get_utf8 (p, &uc))) {
+		if (uc == ENTITY_NBSP || uc == ' ') {
+			rv = check_prev_white (rv, white_space, last_white, delta_out);
+			white_space ++;
+			last_white = uc;
+		} else {
+			rv = check_last_white (rv, white_space, last_white, delta_out);
+			white_space = 0;
+		}
+		op = p;
+	}
+	rv = check_last_white (rv, white_space, last_white, delta_out);
+
+	return rv;
+}
+
+static inline void
+write_prev_white_space (gint white_space, gchar **fill)
+{
+			if (white_space > 0) {
+#ifdef DEBUG_NBSP
+				printf ("&nbsp;");
+#endif
+				**fill = 0xc2; (*fill) ++;
+				**fill = 0xa0; (*fill) ++;
+			}
+}
+
+static inline void
+write_last_white_space (gint white_space, gchar **fill)
+{
+	if (white_space > 0) {
+#ifdef DEBUG_NBSP
+		printf (" ");
+#endif
+		**fill = ' '; (*fill) ++;
+	}
+}
+
 static void
 convert_nbsp (gchar *fill, const gchar *p)
 {
-	gboolean in_white_space;
+	gint white_space;
 	gunichar uc;
 	const gchar *op;
 
 #ifdef DEBUG_NBSP
-	printf ("convert_nbsp: %s --> ", s);
+	printf ("convert_nbsp: %s --> \"", p);
 #endif
 	op = p;
-	in_white_space = FALSE;
+	white_space = 0;
 
 	while (*p && (p = e_unicode_get_utf8 (p, &uc))) {
 		if (uc == ENTITY_NBSP || uc == ' ') {
-			if (in_white_space) {
-#ifdef DEBUG_NBSP
-				printf ("&nbsp;");
-#endif
-				*fill = 0xc2; fill ++;
-				*fill = 0xa0; fill ++;
-			} else {
-#ifdef DEBUG_NBSP
-				printf (" ");
-#endif
-				*fill = ' '; fill++;
-			}
-			in_white_space = TRUE;
+			write_prev_white_space (white_space, &fill);
+			white_space ++;
 		} else {
-			in_white_space = FALSE;
+			write_last_white_space (white_space, &fill);
+			white_space = 0;
 #ifdef DEBUG_NBSP
 			printf ("*");
 #endif
@@ -890,10 +835,11 @@ convert_nbsp (gchar *fill, const gchar *p)
 		op = p;
 	}
 
+	write_last_white_space (white_space, &fill);
 	*fill = 0;
 
 #ifdef DEBUG_NBSP
-	printf ("\n");
+	printf ("\"\n");
 #endif
 }
 
@@ -1066,7 +1012,6 @@ set_color (HTMLText *text,
 	text->color = color;
 
 	if (engine != NULL) {
-		html_object_relayout (HTML_OBJECT (text)->parent, engine, HTML_OBJECT (text));
 		html_engine_queue_draw (engine, HTML_OBJECT (text));
 	}
 }
@@ -1200,9 +1145,15 @@ get_cursor (HTMLObject *self,
 	    gint *x1, gint *y1,
 	    gint *x2, gint *y2)
 {
-	HTMLObject *slave;
+	HTMLObject *slave, *next;
 	guint ascent, descent;
 
+	next = html_object_next_not_slave (self);
+	if (offset == HTML_TEXT (self)->text_len && next && html_object_is_text (next)) {
+		html_object_get_cursor (next, painter, 0, x1, y1, x2, y2);
+		return;
+	}
+
 	html_object_get_cursor_base (self, painter, offset, x2, y2);
 
 	slave = self->next;
@@ -1225,8 +1176,13 @@ get_cursor_base (HTMLObject *self,
 		 guint offset,
 		 gint *x, gint *y)
 {
-	HTMLObject *obj;
-	gint line_offset = html_text_get_line_offset (HTML_TEXT (self), painter);
+	HTMLObject *obj, *next;
+
+	next = html_object_next_not_slave (self);
+	if (offset == HTML_TEXT (self)->text_len && next && html_object_is_text (next)) {
+		html_object_get_cursor_base (next, painter, 0, x, y);
+		return;
+	}
 
 	for (obj = self->next; obj != NULL; obj = obj->next) {
 		HTMLTextSlave *slave;
@@ -1243,17 +1199,18 @@ get_cursor_base (HTMLObject *self,
 			if (offset != slave->posStart) {
 				HTMLText *text;
 				GtkHTMLFontStyle font_style;
+				gint line_offset;
 
 				text = HTML_TEXT (self);
 
 				font_style = html_text_get_font_style (text);
+				line_offset = html_text_slave_get_line_offset (slave,
+									       html_text_get_line_offset (HTML_TEXT (self),
+													  painter),
+									       slave->posStart, painter);
 				*x += html_painter_calc_text_width (painter,
 								    html_text_get_text (text, slave->posStart),
-								    offset - slave->posStart,
-								    html_text_slave_get_line_offset (slave,
-												     line_offset,
-												     slave->posStart,
-												     painter),
+								    offset - slave->posStart, &line_offset,
 								    font_style, text->face);
 			}
 
@@ -1438,6 +1395,7 @@ html_text_set_text (HTMLText *text, const gchar *new_text)
 {
 	g_free (text->text);
 	text->text = g_strdup (new_text);
+	text->text_len = text_len (new_text, -1);
 	html_object_change_set (HTML_OBJECT (text), HTML_CHANGE_ALL);
 }
 
@@ -1633,6 +1591,8 @@ html_text_magic_link (HTMLText *text, HTMLEngine *engine, guint offset)
 		return FALSE;
 	offset--;
 
+	/* printf ("html_text_magic_link\n"); */
+
 	html_undo_level_begin (engine->undo, "Magic link", "Remove magic link");
 	saved_position = engine->cursor->position;
 
@@ -1656,7 +1616,7 @@ html_text_magic_link (HTMLText *text, HTMLEngine *engine, guint offset)
 	}
 
 	html_undo_level_end (engine->undo);
-	html_cursor_jump_to_position (engine->cursor, engine, saved_position);
+	html_cursor_jump_to_position_no_spell (engine->cursor, engine, saved_position);
 
 	return rv;
 }
