@@ -527,10 +527,13 @@ draw_highlighted (HTMLTextSlave *slave,
 
 	/* Draw the highlighted part with a highlight background.  */
 
-	html_painter_set_pen (p, &html_colorset_get_color_allocated (p, HTMLHighlightColor)->color);
+	html_color_alloc (HTML_TEXT (owner)->color, p);
+	html_painter_set_pen (p, &html_colorset_get_color_allocated
+			      (p, p->focus ? HTMLHighlightColor : HTMLHighlightNFColor)->color);
 	html_painter_fill_rect (p, obj->x + tx + offset_width, obj->y + ty - obj->ascent,
 				text_width, obj->ascent + obj->descent);
-	html_painter_set_pen (p, &html_colorset_get_color_allocated (p, HTMLHighlightTextColor)->color);
+	html_painter_set_pen (p, &html_colorset_get_color_allocated
+			      (p, p->focus ? HTMLHighlightTextColor : HTMLHighlightTextNFColor)->color);
 	html_painter_draw_text (p, obj->x + tx + offset_width, 
 				obj->y + ty + get_ys (HTML_TEXT (slave->owner), p),
 				g_utf8_offset_to_pointer (text, start), len,
@@ -538,7 +541,6 @@ draw_highlighted (HTMLTextSlave *slave,
 
 	/* Draw the non-highlighted part.  */
 
-	html_color_alloc (HTML_TEXT (owner)->color, p);
 	html_painter_set_pen (p, &HTML_TEXT (owner)->color->color);
 
 	/* 1. Draw the leftmost non-highlighted part, if any.  */
